==> veracity-source-1.0.0.10517.tar.gz: veracity/src/server_files/ui/burndown.js <==
/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

/*
 * Globals
 */

function burndownController()
{
	this.treeController = null;
	this.summaryController = null;

	this.form = null;
	
	this._defaults = {
		rectype: "burndown",
		sprint: null,
		user: "",
		type: "hourly"
	};
	
	this.graphDiv = $("#burndown");
	this.sprint = null;
	this.burndownChart = null;
	this.initial = true;
	this.ajaxQueue = null;
	
	this.init = function()
	{
		this.burndownChart = new burndownChart($("#burndown"), {
			widthCalculator: function() {
				return $("#maincontent").width();
			},
			heightCalculator: function() {
				var sumHeight = $("#summary").height();
				var cheight = windowHeight() - $("#maincontent").offset().top - 10 - sumHeight;
				return cheight;
			},
			onError: function()
			{
				$("#summary").hide();
			}
		});
		
		this.summaryController = new summaryController();
		this.burndownChart.hideLoading();
		this._drawSidebar();
		this.bindObservers();
		this.fetchChartData();
	};
	
	/**
	 * Create and render the contents of the left hand sidebar
	 */
	this._drawSidebar = function()
	{
		var self = this;
		var vformOptions = {
			rectype: "burndown",
			afterPopulate: function(form) {
				self.formIsReady(form);
			},
			onChange: function(form) {
				self.formDidChange(form);
			}
		};
		
		this.form = new vForm(vformOptions);
		var rec = this._getCookieValues();
		this.form.setRecord(rec);
		
		var header = $("<h1>");
		header.text("Burndown Chart");
		$('#rightSidebar').append(header);

		var xheader = $("<div>").addClass("extendedHeader");
		
		xheader.append(this.form.label("user", "User:"));
		xheader.append(this.form.userSelect("user", {
			allowEmptySelection: true,
			emptySelectionText: "(all users)"
		}));
		
		var options = [
			{ name: "Hour Burndown", value: "hourly"},
			{ name: "Issue Burndown", value: "issue"}
		];
		
		xheader.append(this.form.label("type", "Type:"));
		xheader.append(this.form.select("type", options ));
		
		xheader.append(this.form.label("sprint", "Milestone:"));

		$('#rightSidebar').append(xheader);

		var cont = $("<div>");

		cont.attr("id", "treeControl");
		$('#rightSidebar').append(cont);
		
		this.treeController = new SprintTreeControl($('#treeControl'), { closeReleased: true });
		
		this.form.addCustomField("sprint", this.treeController);
		this.form.ready();
		
	};
	
	/**
	 * Get the cookie values for this page
	 * 
	 * Fetches the cookies to indicate what user and sprint were selected on the
	 * previous page load.
	 */
	this._getCookieValues = function()
	{
		var cname = vCore.getDefaultCookieName();
		var cookie = vCore.getJSONCookie(cname);
		
		var cookievals = {
			rectype: "burndown",
			sprint: (cookie.burndownSprint || sgCurrentSprint),
			user: cookie.burndownUser
		}
		
		var ret = jQuery.extend(true, {}, this._defaults, cookievals);
		return ret;
	},
	
	this._setCookieValues = function()
	{
		var cname = vCore.getDefaultCookieName();
		var rec = this.form.getFieldValues();
		
		var cobj = {
			burndownSprint: rec.sprint,
			burndownUser: rec.user
		}
		
		vCore.setCookieValuesFromObject(cname, cobj, 30)

	},
	
	this.formIsReady = function()
	{
		this.fetchChartData();
	};
	
	this.formDidChange = function(form)
	{
		var newValues = this.form.getFieldValues();
		this._setCookieValues();
		
		if (!newValues.sprint)
			return;
		
		this.sprint = this.treeController.getSprint(newValues.sprint);
		
		if (!this.sprint)
			return;
		this.summaryController.sprint = this.sprint;
		this.summaryController.user = newValues.user;
		this.summaryController.showSummary();
		
		this.burndownChart.burndownType = newValues.type;
		this.burndownChart.user = newValues.user;
		this.burndownChart.options.startDate = new Date(this.sprint.startdate);
		this.burndownChart.options.endDate = new Date(this.sprint.enddate);
		
		if (this.initial || this.form.getRecordValue("sprint") != newValues.sprint)
		{
			this.initial = false;
			this.fetchChartData();
		}
		else
		{
			this.burndownChart.prepareChart();
			this.summaryController.timeline = this.burndownChart.historyTimeline;
			this.summaryController.populate();
		}
		
		this.form.setRecord(newValues);
	};
	
	this.bindObservers = function()
	{
		var self = this;
		// Bind Resize Observers
		$('#maincontent').bind('resize', function() { self.burndownChart.draw(); });
		$('#summary').bind('resize', function() { self.burndownChart.draw(); });
	};
	
	this.fetchChartData = function()
	{
		var self = this;
		var record = this.form.getFieldValues();
		
		if ((!record.sprint) || (record.sprint == '')) {
			self.burndownChart.displayError("Select a sprint to display a burndown");
			return;
		}
		
		var wurl = vCore.getOption("currentRepoUrl") + '/burndown.json?type=sprint&recid=' + record.sprint;

		this.burndownChart.setLoading("Loading Chart Data...");
		this.summaryController.setLoading("Loading Summary Data...");
		
		if (this.ajaxQueue)
			return;
			
		this.ajaxQueue = new ajaxQueue({ 
			onSuccess: function() {
				self.ajaxQueue = null;
				self.burndownChart.prepareChart();
				self.summaryController.timeline = self.burndownChart.historyTimeline;
				self.summaryController.populate();
			}
		});
		
		this.ajaxQueue.addQueueRequest( {
			url : wurl,
			dataType : 'json',
			success : function(data) 
			{
				// TODO find out why the json data is sometimes null
				if (data == null) { self.burndownChart.displayError("An error has occured.  Please refresh the page"); }
				
				self.burndownChart.setChartData(data);
			},
			error : function(xhr, textStatus, errorThrown) 
			{
				vCore.consoleLog("Error: Fetch Chart Data Failed");
				self.burndownChart.displayError("Unable to display chart.");
			}
		});
		
		var wurl = vCore.getOption("currentRepoUrl") + '/milestone/' + record.sprint + "/items.json";
		
		this.ajaxQueue.addQueueRequest( {
			url : wurl,
			dataType : 'json',
			success : function(data) 
			{
				self.summaryController.currentWI = data;
			},
			error : function(xhr, textStatus, errorThrown) 
			{
				vCore.consoleLog("Error: Fetch current Workitems Failed");
			}
		});
		
		this.ajaxQueue.execute();
	};

	this.init();
}

function summaryController()
{
	this.sprint = null;
	this.currentWI = [];
	this.user = null;
	this.timeline = null;
	this.progress = null;
	
	this.init = function()
	{
		var self = this;
		this.progress = new smallProgressIndicator("summaryLoading", "Loading Summary...");
		$("#summary").append(this.progress.draw());
		
		$("#linkhidesum").toggle(function() { self.hideSummary(); }, function() { self.showSummary() });
	}
	
	this.populate = function()
	{
		$("#summary").show();
		
		this.hideLoading();
		
		if (this.sprint.startdate)
			var start = vCore.dateToDayStart(new Date(Math.floor(this.sprint.startdate)));
		if (this.sprint.enddate)
			var end = vCore.dateToDayEnd(new Date(Math.floor(this.sprint.enddate)));
		
		var today = vCore.dateToDayStart(new Date());
		
		var div = $($("#summarylist")[0]);
		div.empty();
		
		var cont = $("<div>").addClass("sprint").addClass("clearfix");
		
		var head = $("<h1>").text(this.sprint.name);
		cont.append(head);
		
		var dates = $("<div>").addClass("sum");
		
		var date_table = $("<table>");
		
		if (start)
			date_table.append(this.tableRow("Start Date",start.strftime("%a %b %d, %Y")));
		if (end)
			date_table.append(this.tableRow("End Date",end.strftime("%a %b %d, %Y")));
		
		if (start && end)
		{
			date_table.append(this.tableRow("Working Days",vCore.workingDays(start, end)));
			if (end.getTime() >= today.getTime())
			{
				var _d = start.getTime() > today.getTime() ? start : today;
				date_table.append(this.tableRow("Working Days Left",vCore.workingDays(_d, end)));
			}
		}
		dates.append(date_table);
		cont.append(dates);
		
		var issues = $("<div>").addClass("sum");
		table = $("<table>");
		
		var open = 0;
		var closed = 0;
		var total = 0;
		for (var i = 0; i < this.currentWI.length; i++)
		{
			var wi = this.currentWI[i];
			if (this.user && this.user != wi.assignee)
				continue;
			
			total += 1;
			if (wi.status == "open")
				open += 1;
			else
				closed += 1;
		}
		
		var totalLink = $("<a>").text(total);
		var url = vCore.repoRoot() + "/workitems.html?milestone=" + this.sprint.recid + "&sort=priority&groupedby=status&columns=id%2Ctitle%2Cassignee%2Creporter%2Cpriority%2Csprint%2Clogged_time%2Cremaining_time&maxrows=100&skip=0";
		if (this.user)
		{
			url += "&assignee=" + this.user;
		}
		totalLink.attr({
			href: url,
			target: "_new"
		});
		table.append(this.tableRow("Total Issues",totalLink));
		
		var closedLink = $("<a>").text(closed);
		var url = vCore.repoRoot() + "/workitems.html?milestone=" + this.sprint.recid + "&status=fixed%2Cverified%2Cwontfix%2Cinvalid%2Cduplicate&sort=priority&groupedby=status&columns=id%2Ctitle%2Cassignee%2Creporter%2Cpriority%2Csprint%2Clogged_time%2Cremaining_time&maxrows=100&skip=0";
		if (this.user)
		{
			url += "&assignee=" + this.user;
		}
		closedLink.attr({
			href: url,
			target: "_new"
		});
		table.append(this.tableRow("Closed Issues",closedLink));
		var openLink = $("<a>").text(open);
		var url = vCore.repoRoot() + "/workitems.html?milestone=" + this.sprint.recid + "&status=open&sort=priority&groupedby=status&columns=id%2Ctitle%2Cassignee%2Creporter%2Cpriority%2Csprint%2Clogged_time%2Cremaining_time&maxrows=100?skip=0";
		if (this.user)
		{
			url += "&assignee=" + this.user;
		}
		openLink.attr({
			href: url,
			target: "_new"
		});
		table.append(this.tableRow("Open Issues",openLink));
		issues.append(table);
		cont.append(issues);
		
		if (this.timeline)
		{
			this.timeline.user = this.user;
			
			var start_log = this.timeline.getLoggedHours(start);
			var start_est = this.timeline.getEstimatedHours(start);
			if (start_est < 0)
				start_est = 0;
				
			var est = this.timeline.getEstimatedHours(end);
			var log = this.timeline.getLoggedHours(end);
			var rem = est - log;
		
			var hours = $("<div>").addClass("sum");
			table = $("<table>");
			table.append(this.tableRow("Starting Estimate",this.scaleTime(start_est - start_log) + "h"));
			table.append(this.tableRow("Total Estimated Work",this.scaleTime(est - start_log) + "h"));
			table.append(this.tableRow("Work Logged",this.scaleTime(log - start_log) + "h"));
			table.append(this.tableRow("Work Remaining",this.scaleTime(rem) + "h"));
			
			if (end && end.getTime() >= today.getTime())
			{
				var required = rem / vCore.workingDays(today, end);
				table.append(this.tableRow("Required Work Per Day",this.scaleTime(required) + "h"));
				
				if ( start.getTime() <= today.getTime() )
				{
					// Figure out the estimate completion date
					var _d = new Date(start.getTime());
					var ticks = [];
					while (_d.getTime() < today.getTime())
					{
						ticks.push(vCore.dateToDayEnd(_d));
						var _d = vCore.nextWorkingDay(_d);
					}
				
					if (ticks.length > 0)
					{
						var last = ticks.length - 1;
						var last_date = vCore.nextWorkingDay(ticks[last]);
						var first = last > 4 ? last - 4 : 0;
						var first_date = vCore.nextWorkingDay(ticks[first]);
						var first_log = this.timeline.getLoggedHours(first_date);
						var first_est = this.timeline.getEstimatedHours(first_date);
						var last_log = this.timeline.getLoggedHours(last_date);
						var last_est = this.timeline.getEstimatedHours(last_date);
						var first_rem = first_est - first_log;
						var last_rem = last_est - last_log
						var rate = (first_rem - last_rem) / 4;
				
						if (rate > 0)
						{
							var rem_working_days = Math.ceil(last_rem / rate);
							var _d = new Date(today.getTime());
							for (var i = 0; i < rem_working_days; i++)
							{
								var _d = vCore.nextWorkingDay(_d);
							}
				
							date_table.append(this.tableRow("Estimated Completion",_d.strftime("%a %b %d, %Y")));
						}
						else
						{
							date_table.append(this.tableRow("Estimated Completion", "N/A"));
						}
					}
				}
			}
			

			
			hours.append(table);
			cont.append(hours);
		}
		div.append(cont);
	};
	
	this.tableRow = function(title, value)
	{
		var tr = $("<tr>");
		tr.append($("<td>").addClass("key").text(title + ": "));
		if (typeof value == "string")
			tr.append($("<td>").css("text-align", "right").text(value));
		else
			tr.append($("<td>").css("text-align", "right").append(value));
			
		return tr;
	}
	
	this.showSummary = function()
	{
		$('#summary #summarylist').show();
		$('#summary').height(180);
		$('#imghidesum').attr("src",
				sgStaticLinkRoot + "/img/opengreen.png");
		$('#summary').trigger('resize');
	};
	
	this.hideSummary = function()
	{
		$('#summary #summarylist').hide();
		$('#summary').height(20);
		$('#imghidesum').attr("src",
				sgStaticLinkRoot + "/img/opengreenleft.png");
		$('#summary').trigger('resize');
	};
	
	this.scaleTime = function(v)
	{
		var scaled = Math.round(v / 15.0) / 4.0;

		return (scaled);
	};
	
	this.setLoading = function()
	{
		$("#summarylist").empty();
		this.progress.setText("Loading Summary...");
		this.progress.show();
	};
	
	this.hideLoading = function()
	{
		this.progress.setText("");
		this.progress.hide();
	}
	
	this.init();
}


function sgKickoff() {
	vCore.showSidebar();
	var controller = new burndownController();
}

==> veracity-source-1.1.0.10565.tar.gz: veracity/src/server_files/ui/burndown.js <==
/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

/*
 * Globals
 */

function burndownController()
{
	this.treeController = null;
	this.summaryController = null;

	this.form = null;
	
	this._defaults = {
		rectype: "burndown",
		sprint: null,
		user: "",
		type: "hourly"
	};
	
	this.graphDiv = $("#burndown");
	this.sprint = null;
	this.burndownChart = null;
	this.initial = true;
	this.ajaxQueue = null;
	
	this.init = function()
	{
		this.burndownChart = new burndownChart($("#burndown"), {
			widthCalculator: function() {
				return $("#maincontent").width();
			},
			heightCalculator: function() {
				var sumHeight = $("#summary").height();
				var cheight = windowHeight() - $("#maincontent").offset().top - 10 - sumHeight;
				return cheight;
			},
			onError: function()
			{
				$("#summary").hide();
			}
		});
		
		this.summaryController = new summaryController();
		this.burndownChart.hideLoading();
		this._drawSidebar();
		this.bindObservers();
		this.fetchChartData();
	};
	
	/**
	 * Create and render the contents of the left hand sidebar
	 */
	this._drawSidebar = function()
	{
		var self = this;
		var vformOptions = {
			rectype: "burndown",
			afterPopulate: function(form) {
				self.formIsReady(form);
			},
			onChange: function(form) {
				self.formDidChange(form);
			}
		};
		
		this.form = new vForm(vformOptions);
		var rec = this._getCookieValues();
		this.form.setRecord(rec);
		
		var header = $("<h1>");
		header.text("Burndown Chart");
		$('#rightSidebar').append(header);

		var xheader = $("<div>").addClass("extendedHeader");
		
		xheader.append(this.form.label("user", "User:"));
		xheader.append(this.form.userSelect("user", {
			allowEmptySelection: true,
			emptySelectionText: "(all users)"
		}));
		
		var options = [
			{ name: "Hour Burndown", value: "hourly"},
			{ name: "Issue Burndown", value: "issue"}
		];
		
		xheader.append(this.form.label("type", "Type:"));
		xheader.append(this.form.select("type", options ));
		
		xheader.append(this.form.label("sprint", "Milestone:"));
		
		$('#rightSidebar').append(xheader);

		var cont = $("<div>");

		cont.attr("id", "treeControl");
		$('#rightSidebar').append(cont);
		
		this.treeController = new SprintTreeControl($('#treeControl'), { closeReleased: true });
		
		this.form.addCustomField("sprint", this.treeController);
		this.form.ready();
		
	};
	
	/**
	 * Get the cookie values for this page
	 * 
	 * Fetches the cookies to indicate what user and sprint were selected on the
	 * previous page load.
	 */
	this._getCookieValues = function()
	{
		var cname = vCore.getDefaultCookieName();
		var cookie = vCore.getJSONCookie(cname);
		
		var cookievals = {
			rectype: "burndown",
			sprint: (cookie.burndownSprint || sgCurrentSprint),
			user: cookie.burndownUser,
		}
		
		var ret = jQuery.extend(true, {}, this._defaults, cookievals);
		return ret;
	},
	
	this._setCookieValues = function()
	{
		var cname = vCore.getDefaultCookieName();
		var rec = this.form.getFieldValues();
		
		var cobj = {
			burndownSprint: rec.sprint,
			burndownUser: rec.user,
		}
		
		vCore.setCookieValuesFromObject(cname, cobj, 30)

	},
	
	this.formIsReady = function()
	{
		this.fetchChartData();
	};
	
	this.formDidChange = function(form)
	{
		var newValues = this.form.getFieldValues();
		this._setCookieValues();
		
		if (!newValues.sprint)
			return;
		
		this.sprint = this.treeController.getSprint(newValues.sprint);
		
		if (!this.sprint)
			return;
		
		var sprintTree = this.treeController.milestoneTree.getSubTree(this.sprint.recid);
		var sprints = sprintTree.root.getData();
		
		this.summaryController.sprint = this.sprint;
		this.summaryController.sprints = sprints;
		this.summaryController.user = newValues.user;
		this.summaryController.showSummary();
		
		this.burndownChart.burndownType = newValues.type;
		this.burndownChart.user = newValues.user;
		this.burndownChart.options.startDate = new Date(this.sprint.startdate);
		this.burndownChart.options.endDate = new Date(this.sprint.enddate);
		
		if (this.initial || this.form.getRecordValue("sprint") != newValues.sprint )
		{
			this.initial = false;
			this.fetchChartData();
		}
		else
		{
			this.burndownChart.prepareChart();
			this.summaryController.timeline = this.burndownChart.historyTimeline;
			this.summaryController.populate();
		}
		
		this.form.setRecord(newValues);
	};
	
	this.bindObservers = function()
	{
		var self = this;
		// Bind Resize Observers
		$('#maincontent').bind('resize', function() { self.burndownChart.draw(); });
		$('#summary').bind('resize', function() { self.burndownChart.draw(); });
	};
	
	this.fetchChartData = function()
	{
		var self = this;
		var record = this.form.getFieldValues();
		
		if ((!record.sprint) || (record.sprint == '')) {
			self.burndownChart.displayError("Select a sprint to display a burndown");
			return;
		}
		
		var wurl = vCore.getOption("currentRepoUrl") + '/burndown.json?type=sprint&recid=' + record.sprint;
			
		this.burndownChart.setLoading("Loading Chart Data...");
		this.summaryController.setLoading("Loading Summary Data...");
		
		if (this.ajaxQueue)
			return;
			
		this.ajaxQueue = new ajaxQueue({ 
			onSuccess: function() {
				self.ajaxQueue = null;
				self.burndownChart.prepareChart();
				self.summaryController.timeline = self.burndownChart.historyTimeline;
				self.summaryController.populate();
			}
		});
		
		this.ajaxQueue.addQueueRequest( {
			url : wurl,
			dataType : 'json',
			success : function(data) 
			{
				// TODO find out why the json data is sometimes null
				if (data == null) { self.burndownChart.displayError("An error has occured.  Please refresh the page"); }
				
				self.burndownChart.setChartData(data);
			},
			error : function(xhr, textStatus, errorThrown) 
			{
				vCore.consoleLog("Error: Fetch Chart Data Failed");
				self.burndownChart.displayError("Unable to display chart.");
			}
		});
		
		this.ajaxQueue.execute();
	};

	this.init();
}

function summaryController()
{
	this.sprint = null;
	this.sprints = [];
	this.user = null;
	this.timeline = null;
	this.progress = null;
	
	this.init = function()
	{
		var self = this;
		this.progress = new smallProgressIndicator("summaryLoading", "Loading Summary...");
		$("#summary").append(this.progress.draw());
		
		$("#linkhidesum").toggle(function() { self.hideSummary(); }, function() { self.showSummary() });
	}
	
	this.populate = function()
	{
		$("#summary").show();
		
		this.hideLoading();
		
		if (this.sprint.startdate)
			var start = vCore.dateToDayStart(new Date(Math.floor(this.sprint.startdate)));
		if (this.sprint.enddate)
			var end = vCore.dateToDayEnd(new Date(Math.floor(this.sprint.enddate)));
		
		var today = vCore.dateToDayStart(new Date());
		
		var div = $($("#summarylist")[0]);
		div.empty();
		
		var cont = $("<div>").addClass("sprint").addClass("clearfix");
		
		var head = $("<h1>").text(this.sprint.name);
		cont.append(head);
		
		var dates = $("<div>").addClass("sum");
		
		var date_table = $("<table>");
		
		if (start)
			date_table.append(this.tableRow("Start Date",start.strftime("%a %b %d, %Y")));
		if (end)
			date_table.append(this.tableRow("End Date",end.strftime("%a %b %d, %Y")));
		
		if (start && end)
		{
			date_table.append(this.tableRow("Working Days",vCore.workingDays(start, end)));
			if (end.getTime() >= today.getTime())
			{
				var _d = start.getTime() > today.getTime() ? start : today;
				date_table.append(this.tableRow("Working Days Left",vCore.workingDays(_d, end)));
			}
		}
		dates.append(date_table);
		cont.append(dates);
		
		var issues = $("<div>").addClass("sum");
		table = $("<table>");

		if (this.timeline)
		{
			var open = this.timeline.getOpenWICount(today);
			var closed = this.timeline.getClosedWICount(today);
			var total = this.timeline.getTotalWICount(today);

			var _sRecids = [];
			$.each(this.sprints, function(i,s) { _sRecids.push(s.recid); });
			var sprintRecids = _sRecids.join("%2C");

			var totalLink = $("<a>").text(total);
			var url = vCore.repoRoot() + "/workitems.html?milestone=" + sprintRecids + "&sort=priority&groupedby=status&columns=id%2Ctitle%2Cassignee%2Creporter%2Cpriority%2Cmilestone%2Clogged_time%2Cremaining_time&maxrows=100&skip=0";
			if (this.user)
			{
				url += "&assignee=" + this.user;
			}
			totalLink.attr({
				href: url,
				target: "_new"
			});
			table.append(this.tableRow("Total Issues",totalLink));

			var closedLink = $("<a>").text(closed);
			var url = vCore.repoRoot() + "/workitems.html?milestone=" + sprintRecids + "&status=fixed%2Cverified%2Cwontfix%2Cinvalid%2Cduplicate&sort=priority&groupedby=status&columns=id%2Ctitle%2Cassignee%2Creporter%2Cpriority%2Cmilestone%2Clogged_time%2Cremaining_time&maxrows=100&skip=0";
			if (this.user)
			{
				url += "&assignee=" + this.user;
			}
			closedLink.attr({
				href: url,
				target: "_new"
			});
			table.append(this.tableRow("Closed Issues",closedLink));
			var openLink = $("<a>").text(open);
			var url = vCore.repoRoot() + "/workitems.html?milestone=" + sprintRecids + "&status=open&sort=priority&groupedby=status&columns=id%2Ctitle%2Cassignee%2Creporter%2Cpriority%2Cmilestone%2Clogged_time%2Cremaining_time&maxrows=100?skip=0";
			if (this.user)
			{
				url += "&assignee=" + this.user;
			}
			openLink.attr({
				href: url,
				target: "_new"
			});
			table.append(this.tableRow("Open Issues",openLink));
			issues.append(table);
			cont.append(issues);
			
			this.timeline.user = this.user;
			
			var start_log = this.timeline.getLoggedHours(start);
			var start_est = this.timeline.getEstimatedHours(start);
			if (start_est < 0)
				start_est = 0;
				
			var est = this.timeline.getEstimatedHours(end);
			var log = this.timeline.getLoggedHours(end);
			var rem = est - log;
		
			var hours = $("<div>").addClass("sum");
			table = $("<table>");
			table.append(this.tableRow("Starting Estimate",this.scaleTime(start_est - start_log) + "h"));
			table.append(this.tableRow("Total Estimated Work",this.scaleTime(est - start_log) + "h"));
			table.append(this.tableRow("Work Logged",this.scaleTime(log - start_log) + "h"));
			table.append(this.tableRow("Work Remaining",this.scaleTime(rem) + "h"));
			
			if (end && end.getTime() >= today.getTime())
			{
				var required = rem / vCore.workingDays(today, end);
				table.append(this.tableRow("Required Work Per Day",this.scaleTime(required) + "h"));
				
				if ( start.getTime() <= today.getTime() )
				{
					// Figure out the estimate completion date
					var _d = new Date(start.getTime());
					var ticks = [];
					while (_d.getTime() < today.getTime())
					{
						ticks.push(vCore.dateToDayEnd(_d));
						var _d = vCore.nextWorkingDay(_d);
					}
				
					if (ticks.length > 0)
					{
						var last = ticks.length - 1;
						var last_date = vCore.nextWorkingDay(ticks[last]);
						var first = last > 4 ? last - 4 : 0;
						var first_date = vCore.nextWorkingDay(ticks[first]);
						var first_log = this.timeline.getLoggedHours(first_date);
						var first_est = this.timeline.getEstimatedHours(first_date);
						var last_log = this.timeline.getLoggedHours(last_date);
						var last_est = this.timeline.getEstimatedHours(last_date);
						var first_rem = first_est - first_log;
						var last_rem = last_est - last_log
						var rate = (first_rem - last_rem) / 4;
				
						if (rate > 0)
						{
							var rem_working_days = Math.ceil(last_rem / rate);
							var _d = new Date(today.getTime());
							for (var i = 0; i < rem_working_days; i++)
							{
								var _d = vCore.nextWorkingDay(_d);
							}
				
							date_table.append(this.tableRow("Estimated Completion",_d.strftime("%a %b %d, %Y")));
						}
						else
						{
							date_table.append(this.tableRow("Estimated Completion", "N/A"));
						}
					}
				}
			}
			

			
			hours.append(table);
			cont.append(hours);
		}
		div.append(cont);
	};
	
	this.tableRow = function(title, value)
	{
		var tr = $("<tr>");
		tr.append($("<td>").addClass("key").text(title + ": "));
		if (typeof value == "string")
			tr.append($("<td>").css("text-align", "right").text(value));
		else
			tr.append($("<td>").css("text-align", "right").append(value));
			
		return tr;
	}
	
	this.showSummary = function()
	{
		$('#summary #summarylist').show();
		$('#summary').height(180);
		$('#imghidesum').attr("src",
				sgStaticLinkRoot + "/img/opengreen.png");
		$('#summary').trigger('resize');
	};
	
	this.hideSummary = function()
	{
		$('#summary #summarylist').hide();
		$('#summary').height(20);
		$('#imghidesum').attr("src",
				sgStaticLinkRoot + "/img/opengreenleft.png");
		$('#summary').trigger('resize');
	};
	
	this.scaleTime = function(v)
	{
		var scaled = Math.round(v / 15.0) / 4.0;

		return (scaled);
	};
	
	this.setLoading = function()
	{
		$("#summarylist").empty();
		this.progress.setText("Loading Summary...");
		this.progress.show();
	};
	
	this.hideLoading = function()
	{
		this.progress.setText("");
		this.progress.hide();
	}
	
	this.init();
}


function sgKickoff() {
	vCore.showSidebar();
	var controller = new burndownController();
}
